package node;
import env.*;
import type.*;
import code.*;
import type.basicType.*;
import text.*;
import java.io.*;

public abstract class Node {

	private Env env;

	public Env env() { return env; }

	public void genEnv( Env env ) {
		this.env = env;
		}
	public abstract String toString();

	protected Type type;
	public Type type() { return type; }

	private Node sibling;
	public Node sibling() { return sibling; }
	public void setSibling( Node sibling ) { this.sibling = sibling; }
		
	public void printStack() {
		Print.error().debugln( "( " + this + " : " + address + " )" );
		if ( sibling != null )
			sibling.printStack();
		}
		
	private Address address;
	public Address address() { return address; }
	public void setAddress( Address address ) { this.address = address; }

	private Usage usage( Usage usage ) {
		Print.error().debugln( "( " + this + " ).usage( " + usage + " )" );
		usage.plus( address );
		if ( sibling == null )
			return usage;
		else
			return sibling.usage( usage );
		}
	
	public Usage usage() {
		return usage( new Usage( env() ) );
		}
		
	protected Register saveReg( int regType, Usage usage ) {
		return null;
		}
	
	public void saveAllReg( Usage usage ) {
		if ( sibling != null ) {
			sibling.saveAllReg( usage );
			}
		saveReg( Usage.INTREG, usage );
		saveReg( Usage.FLOATREG, usage );
		}
	
	private Register deallocReg( int dataType, Usage usage ) {
		if ( sibling != null ) {
			Register register = sibling.deallocReg( dataType, usage );
			if ( register != null )
				return register;
			}
		return saveReg( dataType, usage );
		}
	
	public Register obtainReg( int dataType, Usage usage ) {
		Register register = ( Register ) usage.getFree( dataType );
		if ( register != null )
			return register;
		else
			return deallocReg( dataType, usage );
		}
	
	public Register obtainReg( Type type, Usage usage ) {
		return obtainReg( Usage.dataType( type ), usage );
		}

	public Address obtainTemp( Usage usage ) {
		return usage.getFree( Usage.TEMPMEM );
		}

	public Register loadReg( Usage usage ) {
		Address currAddress = address();
		if ( currAddress instanceof Register )
			return ( Register ) currAddress;
		else if ( currAddress instanceof Displacement ) {
			Register register = obtainReg( type, usage );
			Code.load( type, register, currAddress );
			setAddress( register );
			return register;
			}
		else if ( currAddress instanceof Literal ) {
			Register register = obtainReg( type, usage );
			Literal literal = ( Literal ) currAddress;
			Code.loadImm( type, register, literal );
			setAddress( register );
			return register;
			}
		return null;
		}

	public void loadToReg( IntTempReg register ) {
		Address currAddress = address();
		if ( currAddress instanceof IntTempReg ) {
			IntTempReg intTempReg = ( IntTempReg ) currAddress;
			if ( intTempReg.regNum() == register.regNum() )
				return;
			}
		if ( currAddress instanceof Register ) {
			Register currRegister = ( Register ) currAddress;
			Code.move( currRegister, register );
			}
		else if ( currAddress instanceof Displacement ) {
			Code.load( type, register, currAddress );
			}
		else if ( currAddress instanceof Literal ) {
			Literal literal = ( Literal ) currAddress;
			Code.loadImm( type, register, literal );
			}
		setAddress( register );
		}

	public Displacement loadAddress( Usage usage ) {
		Address currAddress = address();
		if ( currAddress instanceof Displacement ) {
			Displacement displAddr = ( Displacement ) currAddress;
			String offset = displAddr.offset();
			Address base = displAddr.base();
			if ( base instanceof IntReg ) {
				return displAddr;
				}
			Register register = obtainReg( Usage.INTREG, usage );
			if ( base instanceof Displacement )
				Code.load( type, register, base );
			else if ( base instanceof Literal )
				Code.loadImm( type, register, base );
			Displacement newAddress;
			if ( offset.equals( "" ) )
				newAddress = new Indirect( register );
			else
				newAddress = new Displacement( offset, register );
			setAddress( newAddress );
			return newAddress;
			}
		throw new Error( "Error in loadAddress()" );
		}

	}
