package node.declNode;

import grammar.*;
import node.*;
import node.typeNode.*;
import env.*;
import type.*;
import runEnv.*;
import code.*;
import text.*;

import java.util.*;
import java.io.*;

public class ClassDeclNode extends DeclStmtNode implements DeclNode {

	private String ident;
	private String extendsIdent;
	private DeclStmtListNode body;

	private ClassType classType;
	private Decl decl;
	private Decl extendedDecl;
	private Env localEnv;

	public String ident() { return ident; }
	public Decl decl() { return decl; }
	public DeclStmtListNode body() { return body; }

	public ClassDeclNode( 
		String ident,
		String extendsIdent,
		DeclStmtListNode body ) {
		this.ident = ident;
		this.extendsIdent = extendsIdent;
		this.body = body;
		}
	
	public String toString() {
		String extendsText;
		if ( extendsIdent == null )
			extendsText = "";
		else
			extendsText = " extends " + extendsIdent;

		return "class " + ident + extendsText 
			+ "%+%nbegin%+%n" 
			+ body
			+ "%-%nend%-";
		}
		
	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		Env extendedEnv = null;
		if ( extendsIdent != null ) {
			extendedDecl = env.searchEnv( extendsIdent );
			if ( extendedDecl == null )
				throw new Error( "Undeclared class " + extendedDecl );
			else if ( extendedDecl.section() != Decl.DECL_CLASS )
				throw new Error( "extended declaration " + 
					extendedDecl + " should be a class" );
			ClassType extendedClass = ( ClassType ) extendedDecl.type();
			extendedEnv = extendedClass.env();
			}
		localEnv = new Env( Env.ENV_CLASS, env, extendedEnv );
		classType = new ClassType( ident, localEnv );
		decl = env.declare( ident, Decl.DECL_CLASS, this );
		decl.setType( classType );
		localEnv.setDecl( decl );
		body.genEnv( Decl.DECL_VAR, localEnv );
		}

	public void setType() {
		body.setType();
		}

	public void checkType() {
		body.checkType();
		}

	public void genOffset() {
		decl.genOffset();
		localEnv.setClassOffset();
		body.genOffset();
		localEnv.createMethodTable();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		}

	public void genInstanceCode() {
		Code.comment( 
			"Code to initialise an instance of class " + decl.useName() );
		Code.enterBlock( "initInstance", "proc.sav0", "proc" );
		Code.enter( "code" );
		Code.align();
		Code.publicLabelDefn( "enter" );
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "-max", SpecialReg.stackPtr ) );
		Code.storeQ( SpecialReg.retAddr, 
			new Displacement( "savRet", SpecialReg.stackPtr ) );
		Code.storeQ( SpecialReg.framePtr, 
			new Displacement( "savFP", SpecialReg.stackPtr ) );
		Code.storeQ( SpecialReg.instPtr, 
			new Displacement( "savIP", SpecialReg.stackPtr ) );
		Code.move( SpecialReg.newInstPtr, SpecialReg.instPtr );
		Code.move( SpecialReg.stackPtr, SpecialReg.framePtr );
		if ( extendedDecl != null )
			Code.bsr( new Relative( 
				extendedDecl.absoluteName() + ".initInstance.enter" ) );
		body.genCode();
		Code.labelDefn( "return" );
		Code.move( SpecialReg.instPtr, SpecialReg.newInstPtr );
		Code.loadQ( SpecialReg.instPtr, 
			new Displacement( "savIP", SpecialReg.stackPtr ) );
		Code.loadQ( SpecialReg.framePtr, 
			new Displacement( "savFP", SpecialReg.stackPtr ) );
		Code.loadQ( SpecialReg.retAddr, 
			new Displacement( "savRet", SpecialReg.stackPtr ) );
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "+max", SpecialReg.stackPtr ) );
		Code.ret();
		Code.exit( "code" );
		localEnv.genTempSpace();
		Code.exitBlock( "initInstance" );
		}

	public void genDeclCode() {
		Code.enterBlock( decl.declName(), null, null );
		localEnv.genFieldDeclOffsetCode();
		localEnv.genMethodDeclOffsetCode();
		localEnv.genMethodTable();
		localEnv.genMethodDeclCode();
		genInstanceCode();
		Code.exitBlock( decl.declName() );
		}

	public void genCode() {
		}
	}
