package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;
import text.*;

public class AssignNode extends ExprNode {

	private VariableNode variable;
	private ExprNode expr;
	private Type varType;
	
	public AssignNode( VariableNode variable, ExprNode expr ) {
		this.variable = variable;
		this.expr = expr;
		precedence = PREC_ASSIGN;
		}
	
	public String toString() {
		return variable + " = " + expr;
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		variable.genEnv( env );
		expr.genEnv( env );
		}

	public Type checkType() {
		varType = variable.checkType();
		if ( ! variable.addressable() )
			throw new Error( "Operand for = not addressable" );
		Type exprType = expr.checkType();
		expr = expr.castTo( Type.CAST_IMPLICIT, varType );
		type = VoidType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		PtrValue address = variable.evalAddr( runEnv );
		RunValue value = expr.eval( runEnv );
		address.setValue( value );
		return VoidValue.value;
		}
		
	public void evalCode( Node sibling ) {
		setSibling( sibling );
		expr.evalCode( sibling );
		variable.evalAddrCode( expr );
		expr.loadReg( usage().plus( variable.address() ) );
		variable.loadAddress( usage().plus( expr.address() ) );
		setAddress( null );
		Code.store( varType, ( Register ) expr.address(), variable.address() );
		}
	}
