package node.exprNode;
import grammar.*;
import node.*;
import node.exprNode.castNode.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

import java.io.*;

public abstract class ExprNode extends Node {

	public final static int PREC_ASSIGN		=	2;
	public final static int PREC_OR			=	4;
	public final static int PREC_AND		=	5;
	public final static int PREC_EQ			=	9;
	public final static int PREC_REL		=	9;
	public final static int PREC_ADD		=	12;
	public final static int PREC_MUL		=	13;
	public final static int PREC_CAST		=	15;
	public final static int PREC_PREFIX		=	15;
	public final static int PREC_POSTFIX	=	16;
	public final static int PREC_PRIMARY	=	17;

	protected int precedence;

	public String toString( int parentPrec ) {
		if ( precedence < parentPrec )
			return "( " + toString() + " )";
		else
			return toString();
		}

	public abstract Type checkType();
	
	public abstract RunValue eval( RunEnv runEnv );

	public ExprNode castTo( int castKind, Type postType ) {
		ExprNode exprNode = null;
		if ( ! postType.isCastable( castKind, type ) )
			throw new Error( "Can't cast " + type + " to " + postType );
		if ( type.equals( postType ) ) {
			return this;
			}
		else if ( postType instanceof VoidType ) {
			exprNode = new CastToVoidNode( this );
			}
		else if ( type instanceof IntType && postType instanceof CharType ) {
			exprNode = this;
			}
		else if ( type instanceof CharType && postType instanceof IntType ) {
			exprNode = this;
			}
		else if ( postType instanceof PtrType 
			&& postType.isCastable( Type.CAST_IMPLICIT, type ) ) {
			exprNode = this;
			}
		else {
			throw new Error( "Can't cast " + type + " to " + postType );
			}
		return exprNode;
		}
	
	public abstract void evalCode( Node sibling );
		
	public void boolCode(
		Node sibling,
		Relative trueLabel, 
		Relative falseLabel, 
		boolean flowIntoTrue ) {
		evalCode( sibling );
		Register resultReg = loadReg( new Usage( env() ) );
		if ( flowIntoTrue ) {
			Code.instrn( "blbc", resultReg, falseLabel );
			}
		else {
			Code.instrn( "blbs", resultReg, trueLabel );
			}
		}

	public Register saveReg( int dataType, Usage usage ) {
		Address currAddress = address();
		if ( dataType == Usage.INTREG && currAddress instanceof IntTempReg ) {
			Register register = ( Register ) currAddress;
			usage.minus( currAddress );
			Address newAddress = obtainTemp( usage );
			Code.store( type, register, newAddress );
			setAddress( newAddress );
			return register;
			}
		else if ( dataType == Usage.FLOATREG && currAddress instanceof FloatTempReg ) {
			Register register = ( Register ) currAddress;
			usage.minus( currAddress );
			Address newAddress = obtainTemp( usage );
			Code.store( type, register, newAddress );
			setAddress( newAddress );
			return register;
			}
		else if ( dataType == Usage.INTREG && currAddress instanceof Displacement ) {
			Displacement displAddr = ( Displacement ) currAddress;
			Address base = displAddr.base();
			if ( ! ( base instanceof IntTempReg ) )
				return null;
			Register register = ( Register ) base;
			Code.load( type, register, currAddress );
			usage.minus( currAddress );
			Address newAddress = obtainTemp( usage );
			Code.store( type, register, newAddress );
			setAddress( newAddress );
			return register;
			}
		return null;
		}
	
	}
