package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class DoStmtNode extends StmtNode {

	private DeclStmtListNode stmtList;
	private ExprNode cond;

	public DoStmtNode( DeclStmtListNode stmtList, ExprNode cond ) {
		this.stmtList = stmtList;
		this.cond = cond;
		}

	public String toString() {
		return "do%+%n" + stmtList + "%-%nend%nwhile " + cond + ";";
		}

	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		stmtList.genEnv( section, env );
		cond.genEnv( env );
		}

	public void checkType() {
		stmtList.checkType();
		Type condType = cond.checkType();
		cond = cond.castTo( Type.CAST_IMPLICIT, BoolType.type );
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		do
			stmtList.eval( runEnv );
		while ( cond.eval( runEnv ).boolValue() );
		}

	public void genCode() {
		Code.enter();
		Code.labelDefn( "do" );
		stmtList.genCode();
		Code.labelDefn( "continue" );
		cond.boolCode( null, new Relative( "do" ), new Relative( "end" ), false );
		Code.labelDefn( "break" );
		Code.labelDefn( "end" );
		Code.exit();
		}
	}
