package node.stmtNode;
import node.*;
import node.exprNode.*;
import node.stmtNode.elseOptNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class IfStmtNode extends StmtNode {

	private ExprNode cond;
	private DeclStmtListNode thenPart;
	private ElseOptNode elsePart;

	public IfStmtNode( 
		ExprNode cond, 
		DeclStmtListNode thenPart, 
		ElseOptNode elsePart ) {
		this.cond = cond;
		this.thenPart = thenPart;
		this.elsePart = elsePart;
		}

	public String toString() {
		return "if " + cond + " then%+%n" 
			+ thenPart + "%-" + elsePart + "%nend";
		}

	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		cond.genEnv( env );
		thenPart.genEnv( section, env );
		elsePart.genEnv( section, env );
		}

	public void checkType() {
		Type condType = cond.checkType();
		cond = cond.castTo( Type.CAST_IMPLICIT, BoolType.type );
		thenPart.checkType();
		elsePart.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		if ( cond.eval( runEnv ).boolValue() )
			thenPart.eval( runEnv );
		else
			elsePart.eval( runEnv );
		}

	public void genCode() {
		Code.enter();
		Code.labelDefn( "if" );
		cond.boolCode( null, 
			new Relative( "then" ), new Relative( "else" ), true );
		Code.labelDefn( "then" );
		thenPart.genCode();
		Code.br( new Relative( "end" ) );
		Code.labelDefn( "else" );
		elsePart.genCode();
		Code.labelDefn( "end" );
		Code.exit();
		}
	}
