package node.stmtNode;
import node.*;
import node.exprNode.*;
import node.declNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class ReturnExprStmtNode extends StmtNode {

	private ExprNode expr;
	private Decl methodDecl;
	private MethodDeclNode methodDeclNode;
	private MethodType methodType;
	private Type returnType;

	public ReturnExprStmtNode( ExprNode expr ) {
		this.expr = expr;
		}

	public String toString() {
		return "return " + expr +";";
		}

	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		expr.genEnv( env );
		}

	public void checkType() {
		Type exprType = expr.checkType();
		methodDecl = env().methodDecl();
		if ( methodDecl == null )
			throw new Error( "Return not inside method" );
		methodDeclNode = ( MethodDeclNode ) methodDecl.declNode();
		methodType = ( MethodType ) methodDecl.type();
		returnType = methodType.returnType();
		expr = expr.castTo( Type.CAST_IMPLICIT, returnType );
		}
		

	public void eval( RunEnv runEnv ) throws ReturnException {
		RunValue result = expr.eval( runEnv );
		throw new ReturnException( result );
		}

	public void genCode() {
		Code.enter();
		expr.evalCode( null );
		expr.loadToReg( ( IntTempReg ) 
			( new Usage( env() ).getFree( returnType ) ) );
		Code.br( new Relative( "return" ) );
		Code.exit();
		}
	}
