package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import code.*;

public class WhileStmtNode extends StmtNode {

	private ExprNode cond;
	private DeclStmtListNode stmtList;

	public WhileStmtNode( ExprNode cond, DeclStmtListNode stmtList ) {
		this.cond = cond;
		this.stmtList = stmtList;
		}

	public String toString() {
		return "while " + cond + " do%+%n" + stmtList + "%-%nend";
		}

	public void genEnv( int section, Env env ) {
		super.genEnv( section, env );
		cond.genEnv( env );
		stmtList.genEnv( section, env );
		}

	public void checkType() {
		Type condType = cond.checkType();
		cond = cond.castTo( Type.CAST_IMPLICIT, BoolType.type );
		stmtList.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		while ( cond.eval( runEnv ).boolValue() )
			stmtList.eval( runEnv );
		}

	public void genCode() {
		Code.enter();
		Code.labelDefn( "while" );
		cond.boolCode( null, 
			new Relative( "do" ), new Relative( "end" ), true );
		Code.labelDefn( "do" );
		stmtList.genCode();
		Code.labelDefn( "continue" );
		Code.br( new Relative( "while" ) );
		Code.labelDefn( "break" );
		Code.labelDefn( "end" );
		Code.exit();
		}
	}
