package type;

import type.basicType.*;
import env.*;
import runEnv.*;
import runEnv.basicValue.*;
import code.*;

public  class ArrayType extends Type {

	private int numElements;
	private Type subType;
	public int numElements() { return numElements; }
	public Type subType() { return subType; }
	
	public ArrayType( int numElements, Type subType ) {
		this.numElements = numElements;
		this.subType = subType;
		}
	
	public Type formalType() {
		return new PtrType( subType );
		}

	public boolean equals( Type type ) {
		if ( ! ( type instanceof ArrayType ) )
			return false;
		ArrayType otherType = ( ArrayType ) type;
		return numElements == otherType.numElements() 
			&& subType().equals( otherType.subType() );
		}

	public boolean isCastable( int castKind, Type type ) {
		return equals( type );
		}

	public String toString() {
		return "[ " + numElements + " ]" + subType;
		}
		
	public Offset size() {
		Offset elementSize = subType.size();
		return new Offset( 
			numElements * elementSize.interp(), 
			numElements * elementSize.compile() );
		}
		
	public void initAddress( RunEnv runEnv, PtrValue address ) 
		throws ReturnException {
		int elementSize = subType.size().interp();
		for ( int i = 0; i < numElements; i++ ) {
			subType.initAddress( runEnv, 
				address.elementAt( elementSize * i ) );
			}
		}

	public Type primitiveType() {
		return subType.primitiveType();
		}

	public int numPrimitiveElements() { 
		return numElements * subType.numPrimitiveElements(); 
		}

	public void initAddressCode() {
		Code.enter();
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "-8", SpecialReg.stackPtr ) );
		Code.storeQ( SpecialReg.s0, new Indirect( SpecialReg.stackPtr ) );
		Code.labelDefn( "for" );
		Code.loadImm( IntType.type, SpecialReg.s0, numPrimitiveElements() );
		Code.labelDefn( "while" );
		Code.instrn( "ble", SpecialReg.s0, new Relative( "end" ) );
		primitiveType().initAddressCode();
		Code.instrn( "addq", SpecialReg.newInstPtr, 
			new Literal( primitiveType().size().compile() ) );
		Code.instrn( "subq", SpecialReg.s0, new Literal( "1" ) );
		Code.instrn( "br", new Relative( "while" ) );
		Code.labelDefn( "end" );
		Code.loadQ( SpecialReg.s0, new Indirect( SpecialReg.stackPtr ) );
		Code.loadA( SpecialReg.stackPtr, 
			new Displacement( "+8", SpecialReg.stackPtr ) );
		Code.exit();
		}
	}
