package node.exprNode;

import runEnv.*;
import node.stmtNode.*;

import java.util.*;

public class InvocationNode extends ExprNode {

	private String ident;
	private ExprListNode actualParamList;

	public InvocationNode( String ident, ExprListNode actualParamList ) {
		this.ident = ident;
		this.actualParamList = actualParamList;
		precedence = PREC_PRIMARY;
		}
	
	public String toString() {
		return ident + "( " + actualParamList + " )";
		}

	public Object eval( RunEnv runEnv ) {
		FunctionValue functionValue = Cast.functionValue( runEnv.search( ident ) );
		RunEnv globals = functionValue.globals();
		Vector< Object > actualParams = actualParamList.eval( runEnv );
		FormalParamListNode formalParams = functionValue.formalParamList();
		Hashtable< String, Object > localDecls = new Hashtable< String, Object >();
		if ( formalParams.size() != actualParams.size() )
			throw new Error( "Wrong number of actual parameters" );
		for ( int i = 0; i < formalParams.size(); i++ ) {
			String ident = formalParams.elementAt( i );
			if ( localDecls.get( ident ) != null )
				throw new Error( ident + " redeclared" );
			localDecls.put( ident, actualParams.elementAt( i ) );
			}
		return functionValue.expr().eval( 
			new RunEnv( localDecls, functionValue.globals() ) );
		}
	}
