package env;

import type.*;
import node.*;
import node.declNode.*;

public class Decl {

	public final static int NONE = 0;
	public final static int TYPE = 1;
	public final static int VAR = 2;
	public final static int FUNCT = 3;
	public final static int FIELD = 4;
	public final static int VALUEPARAM = 5;
	public final static int VARPARAM = 6;

	public static String kindText( int kind ) {
		switch ( kind ) {
			case TYPE:
				return "type";
			case VAR:
				return "var";
			case FUNCT:
				return "funct";
			case VARPARAM:
				return "var param";
			case FIELD:
				return "field";
			case VALUEPARAM:
				return "value param";
			default:
				throw new Error( "Invalid kind in declaration" );
			}
		}

	private String ident;
	private int kind;
	private int offset;
	private int level;
	private Type type;
	private BlockNode body;

	public String ident() { return ident; }
	public int kind() { return kind; }
	public int offset() { return offset; }
	public int level() { return level; }
	public Type type() { 
		if ( type == null )
			return null;
		else
			return type.nonRefType();
		}

	public BlockNode body() { return body; }

	public Decl( String ident, int kind, Type type ) {
		this.ident = ident;
		this.kind = kind;
		this.type = type;
		this.body = new ErrorBlockNode();
		}

	public void setType( Type type ) {
		this.type = type; // Can be RefType
		}

	public void setLevel( int level ) {
		//	Print.error().println( "setLevel " + ident + " " + level );
		this.level = level;
		}

	public void setOffset( int offset ) {
		this.offset = offset;
		}

	public void setBody( BlockNode body ) {
		this.body = body;
		}

	public String toString() {
		switch ( kind ) {
			case TYPE:
				return "type " + ident + " = " + type + ";"
					+ "%+%nlevel " + level + ", offset " + offset + "%-";
			case FUNCT:
				return "funct " + ident + type + ";"
					+ "%+%nlevel " + level + ", offset " + offset + "%-";
			case VAR:
			case VARPARAM:
				return "var " + ident + ":" + type + ";"
					+ "%+%nlevel " + level + ", offset " + offset + "%-";
			case FIELD:
			case VALUEPARAM:
				return ident + ":" + type + ";"
					+ "%+%nlevel " + level + ", offset " + offset + "%-";
			default:
				throw new Error( "Invalid kind in declaration" );
			}
		}
	}
