package node.exprNode.binaryNode.arithNode;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import node.exprNode.*;

public class PlusNode extends ArithNode {

	public PlusNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_ADD;
		operator = "+";
		}

	public Type checkType( Env env ) {
		Type leftType = left.checkType( env );
		Type rightType = right.checkType( env );
		Type operandType;
		if ( leftType instanceof StringType
			|| rightType instanceof StringType )
			operandType = StringType.type;
		else
			operandType = Type.balanceType( leftType, rightType );
		if ( operandType == null )
			error( "Incompatible operand types of " + leftType + 
				" and " + rightType + " for relation" );
		left = left.castTo( operandType );
		right = right.castTo( operandType );
		type = operandType;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( type instanceof IntType )
			return new RunValue( leftValue.intValue() + rightValue.intValue() );
		else if ( type instanceof RealType )
			return new RunValue( leftValue.realValue() + rightValue.realValue() );
		else if ( type instanceof StringType )
			return new RunValue( leftValue.stringValue() + rightValue.stringValue() );
		error( "Invalid Type for +" );
		return null;
		}
	}
