package node.exprNode.primaryNode;
import env.*;
import node.exprNode.*;
import type.*;
import runEnv.*;

public class InvocationNode extends PrimaryNode {

	private String ident;
	private ExprListNode actualParamList;
	private Decl decl;
	private int level;

	public InvocationNode( String ident, ExprListNode actualParamList ) {
		this.ident = ident;
		this.actualParamList = actualParamList;
		}
	
	public String toString() {
		return ident + "( " + actualParamList + " )";
		}

	public Type checkType( Env env ) {
		level = env.level();
		decl = env.search( ident );
		if ( decl == null )
			error( "Undeclared function " + ident );
		if ( decl.kind() != Decl.FUNCT )
			error( "Invocation of non-function" );
		FunctType functType = ( FunctType ) decl.type();
		actualParamList.checkType( env );
		actualParamList.castParams( functType );
		type = functType.returnType();
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		//	Print.error().println( "Computing actual params" );
		Multiple actualParams = actualParamList.evalParams( 
			runEnv, ( FunctType ) decl.type() );
		try {
			//	Print.error().println( "Computing local environment" );
			RunEnv localEnv = new RunEnv( actualParams, runEnv.staticLink( level - decl.level() ) );
			//	Print.error().println( "Computing body" );
			decl.body().eval( localEnv );
			//	Print.error().println( "Completed" );
			}
		catch ( ReturnException exception ) {
			return exception.returnValue();
			}
		return new RunValue();
		}
	}
