import java.io.*;
import java_cup.runtime.*;
import node.*;
import env.*;
import runEnv.*;
import text.*;
import grammar.*;

public class Main {

	public static void main( String[] argv ) {
		String dirName = null;

		try {
			for ( int i = 0; i < argv.length; i++ ) {
				if ( argv[ i ].equals( "-debug" ) ) {
					Print.DEBUG = true;
					}
				else if ( argv[ i ].equals( "-dir" ) ) {
					i++;
					if ( i >= argv.length )
						throw new Error( "Missing directory name" );
					dirName = argv[ i ];
					}
				else {
					throw new Error( 
						"Usage: java Main [-debug] -dir directory" );
					}
				}

			if ( dirName == null )
				throw new Error( "Directory not specified" );

			Print.setError( new File( dirName, "program.err" ) );
			Print.setReprint( new File( dirName, "program.print" ) );
			Print.setInterp( new File( dirName, "program.out" ) );

			parser libraryParser = new parser( 
				new File( "LIBRARY", "globalLibrary.in" ) );
			DeclStmtListNode globalLibrary = 
				( DeclStmtListNode ) ( libraryParser.parse().value );
			
			parser programParser = new parser( 
				new File( dirName, "program.in" ) );
			DeclStmtListNode declStmtList; 
			if ( Print.DEBUG )
				declStmtList = 
					( DeclStmtListNode ) programParser.debug_parse().value;
			else
				declStmtList = 
					( DeclStmtListNode ) programParser.parse().value;

			ProgramNode program = new ProgramNode( globalLibrary, declStmtList );

			Print.error().println( "Reprinting ... " );
			Print.reprint().println( program );
			Print.error().println( "Generate Environments ... " );
			Env env = new Env( Env.ENV_GLOBAL );
			program.genEnv( env );
			Print.error().println( "Set Type ... " );
			program.setType();
			Print.error().println( "Check Types ... " );
			program.checkType();
			Print.error().println( "Evaluate ... " );
			try {
				RunEnv runEnv = new RunEnv( new Multiple( env.offset().value() ), null );
				program.eval( runEnv );
				}
			catch ( Error exception ) {
				Print.error().println( "User Error " + exception.getMessage() );
				Print.error().printStackTrace( exception );
				}			
			}
		catch ( Throwable exception ) {
			exception.printStackTrace();
			Print.error().println( "Exception in Main " + exception );
			Print.error().printStackTrace( exception );
			System.exit( -1 );
			}
		}
	}

