package env;

import node.*;
import node.declNode.*;
import type.*;
import text.*;

public class Decl {

	public final static int SECT_NONE		= 0;
	public final static int SECT_VALUEPARAM	= 1;
	public final static int SECT_VARPARAM	= 2;
	public final static int SECT_VARIABLE	= 3;
	public final static int SECT_METHOD		= 4;
	public final static int SECT_CLASS		= 5;
	public final static int SECT_FIELD		= 6;
	public final static int SECT_LABEL		= 7;

	public static String sectionText( int section ) {
		switch ( section ) {
			case SECT_VALUEPARAM:
				return "value param";
			case SECT_VARPARAM:
				return "var param";
			case SECT_VARIABLE:
				return "variable";
			case SECT_METHOD:
				return "method";
			case SECT_CLASS:
				return "type";
			case SECT_FIELD:
				return "field";
			case SECT_LABEL:
				return "label";
			default:
				return "section_" + section;
			}
		}

	private String ident;
	private int section;
	private int offset;
	private Type type;
	private DeclNode declNode;
	private Env env;

	public String ident() { return ident; }
	public int section() { return section; }
	public int offset() { return offset; }
	public Type type() { return type; }
	public DeclNode declNode() { return declNode; }
	public Env env() { return env; }
	public int nestingLevel() { return env.nestingLevel(); }

	public Decl( 
		String ident, 
		int section, 
		DeclNode declNode, 
		Env env ) {
		this.ident = ident;
		this.section = section;
		this.declNode = declNode;
		this.env = env;
		}

	public void setType( Type type ) {
		this.type = type;
		switch ( section ) {
			case SECT_VARPARAM:
			case SECT_VALUEPARAM:
			case SECT_VARIABLE:
			case SECT_FIELD:
				this.offset = env.offset().inc( 1 );
				break;
			}
		}

	public String toString() { return ident; }
	}
