package node.exprNode.binaryNode.arithNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class PlusNode extends ArithNode {

	public PlusNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_ADD;
		operator = "+";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		if ( leftType instanceof StringType
			|| rightType instanceof StringType ) {
			type = StringType.type;
			left = left.castTo( type );
			right = right.castTo( type );
			return type;
			}
		else {
			type = Type.arithBalanceType( leftType, rightType );
			if ( type == null )
				error( "Incompatible operand types of " + leftType + 
					" and " + rightType + " for +" );
			left = left.castTo( type );
			right = right.castTo( type );
			return type;
			}
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( type instanceof IntType )
			return new IntValue( leftValue.intValue() + rightValue.intValue() );
		else if ( type instanceof RealType )
			return new RealValue( leftValue.realValue() + rightValue.realValue() );
		else if ( type instanceof StringType )
			return new StringValue( "" + leftValue + rightValue );
		error( "Invalid Type for +" );
		return null;
		}

	}
