package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.declNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import text.*;

public class InvocationNode extends PrimaryNode {

	private String ident;
	private ExprListNode actualParamList;
	private MethodType methodType;
	private Decl decl;

	public InvocationNode( String ident, ExprListNode actualParamList ) {
		this.ident = ident;
		this.actualParamList = actualParamList;
		}
	
	public String toString() {
		return ident + "( " + actualParamList + " )";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		actualParamList.genEnv( env );
		}

	public Type checkType() {
		decl = env().searchEnv( ident );
		if ( decl == null )
			error( "undeclared identifier " + ident );
		switch ( decl.section() ) {
			case Decl.SECT_METHOD:
				methodType = (MethodType ) decl.type();
				break;
			default:
				error( "Attempting to invoke non-method" );
			}
		actualParamList.checkType();
		actualParamList.castParams( methodType );
		type = methodType.returnType();
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		MethodDeclNode methodDeclNode = ( MethodDeclNode ) decl.declNode();
		Env localEnv = methodDeclNode.localEnv();
		int offset = localEnv.offset().value();
		Multiple locals = new Multiple( offset );
		Multiple evalParams = actualParamList.evalParams( runEnv, methodType );
		for ( int i = 0; i < actualParamList.size(); i++ )
			locals.setValue( i, evalParams.getValue( i ) );
		try {
			RunEnv invocationRunEnv = 
				new RunEnv( locals, runEnv.enclosingEnv( decl.nestingLevel() ) );
			methodDeclNode.body().eval( invocationRunEnv );
			if ( type instanceof VoidType )
				return VoidValue.value;
			else
				throw new Error( "Method did not execute return statement" );
			}
		catch ( ReturnException exception ) {
			return exception.returnValue();
			}
		catch ( BreakException exception ) {
			throw new Error( "Break statement not inside loop" );
			}
		catch ( ContinueException exception ) {
			throw new Error( "Continue statement not inside loop" );
			}
		catch ( UserException exception ) {
			throw new Error( "User Exception " + exception );
			}
		}

	}
