package node.exprNode.primaryNode;
import node.*;
import node.typeNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class SubArrayNode extends PrimaryNode {

	private ExprNode arrayExpr;
	private ExprNode lowExpr;
	private ExprNode highExpr;

	public SubArrayNode( 
		ExprNode arrayExpr,
		ExprNode lowExpr,
		ExprNode highExpr ) {
		this.arrayExpr = arrayExpr;
		this.lowExpr = lowExpr;
		this.highExpr = highExpr;
		}

	public String toString() {
		return arrayExpr.toString( ExprNode.PREC_PRIMARY ) 
			+ "[ " + lowExpr + " .. " + highExpr + " ]";
		}

	public void genEnv( Env env ) {
		super.genEnv( env );
		arrayExpr.genEnv( env );
		lowExpr.genEnv( env );
		highExpr.genEnv( env );
		}

	public Type checkType() {
		Type arrayExprType = arrayExpr.checkType();
		Type lowExprType = lowExpr.checkType();
		Type highExprType = highExpr.checkType();
		lowExpr = lowExpr.castTo( IntType.type );
		highExpr = highExpr.castTo( IntType.type );
		type = arrayExprType;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue arrayValue = arrayExpr.eval( runEnv );
		RunValue lowValue = lowExpr.eval( runEnv );
		RunValue highValue = highExpr.eval( runEnv );
		int offset = lowValue.intValue();
		int size = highValue.intValue() - lowValue.intValue() + 1;
		if ( arrayValue instanceof NullValue )
			throw new Error( "Attempting to form subarray of null array" );
		return arrayValue.arrayValue().subArray( offset, size );
		}

	}
