package node;
import env.*;
import runEnv.*;
import text.*;
import java.io.*;

public abstract class Node {
	public static final int MAXREG = 10;
	public abstract String toString();

	public void error( String s ) throws Error {
		throw new Error( "Error: " + s + " at%+%n" + this + "%-" );
		}

	public static int max( int x, int y ) {
		return Math.max( x, y );
		}

	public static int max( int x, int y, int z ) {
		return Math.max( x, Math.max( y, z ) );
		}

	public static int max( int w, int x, int y, int z ) {
		return Math.max( w, Math.max( x, Math.max( y, z ) ) );
		}

	public static int min( int x, int y ) {
		return Math.min( x, y );
		}

	public static String tempReg( int i ) {
		return "$t" + i;
		}

	public static String ftempReg( int i ) {
		return "$ft" + i;
		}

	public static String displacement( int offset, String regName ) {
		return offset + "(" + regName + ")";
		}

	public static String displacement( String offset, String regName ) {
		return offset + "(" + regName + ")";
		}

	public static String indirect( String regName ) {
		return displacement( 0, regName );
		}

	public static String tempMem( int i ) {
		return displacement( "sav" + i, "$fp" );
		}

	public static String savRA( String regName ) {
		return displacement( "savRA", regName );
		}

	public static String savFP( String regName ) {
		return displacement( "savFP", regName );
		}

	public static String savIP( String regName ) {
		return displacement( "savIP", regName );
		}

	public static String literal( boolean b ) {
		if ( b )
			return "1";
		else
			return "0";
		}

	public static String literal( int i ) {
		return "" + i;
		}

	public static String literal( double d ) {
		return "" + d;
		}

	public static void enter() {
		Print.compile().println( "{%+" );
		}

	public static void enter( String segment ) {
		Print.compile().println( segment + " {%+" );
		}

	public static void enter( String segment, String extending ) {
		Print.compile().println( segment + " extends " + extending + " {%+" );
		}

	public static void enter( String segment, int extending ) {
		Print.compile().println( segment + " extends " + Integer.toString( extending ) + " {%+" );
		}

	public static void enterBlock( String name, String extending, String using ) {
		String result = "public block " + name;
		if ( extending != null )
			result += " extends " + extending;
		if ( using != null )
			result += " uses " + using;
		result += " {%+";
		Print.compile().println( result );
		}

	public static void exit() {
		Print.compile().println( "%-}" );
		}

	public static void instrn( String opCode ) {
		Print.compile().println( opCode + ";" );
		}

	public static void instrn( String opCode, String opd1 ) {
		Print.compile().println( opCode + " " + opd1 + ";" );
		}

	public static void instrn( String opCode, String opd1, String opd2 ) {
		Print.compile().println( opCode + " " + opd1 + ", " + opd2 + ";" );
		}

	public static void instrn( String opCode, String opd1, String opd2, String opd3 ) {
		Print.compile().println( opCode + " " + opd1 + ", " + opd2 + ", " + opd3 + ";" );
		}

	public static void load( String regName, String memory ) {
		instrn( "ldq", regName, memory );
		}

	public static void loadF( String regName, String memory ) {
		instrn( "ldt", regName, memory );
		}

	public static void loadA( String regName, String memory ) {
		instrn( "lda", regName, memory );
		}

	public static void loadIQ( String regName, String value ) {
		instrn( "ldiq", regName, value );
		}

	public static void loadIF( String regName, String value ) {
		instrn( "ldit", regName, value );
		}

	public static void store( String regName, String memory ) {
		instrn( "stq", regName, memory );
		}

	public static void storeF( String regName, String memory ) {
		instrn( "stt", regName, memory );
		}

	public static void move( String source, String dest ) {
		instrn( "mov", source, dest );
		}

	public static void move( int source, String dest ) {
		instrn( "mov", "0x" + Integer.toHexString( source ), dest );
		}

	public static void moveF( String source, String dest ) {
		instrn( "fmov", source, dest );
		}

	public static void jsr() {
		instrn( "jsr", "$ra", "($pv)" );
		}

	public static void jsr( String address ) {
		instrn( "ldiq", "$pv", address );
		jsr();
		}

	public static void bsr( String address ) {
		instrn( "bsr", address );
		}

	public static void br( String address ) {
		instrn( "br", address );
		}

	public static void clear( String regName ) {
		instrn( "clr", regName );
		}

	public static void clearF( String regName ) {
		instrn( "fclr", regName );
		}

	public static void ret() {
		instrn( "ret" );
		}

	public static void real( String data ) {
		instrn( "double", data );
		}

	public static void quad() {
		instrn( "quad" );
		}

	public static void quad( String data ) {
		instrn( "quad", data );
		}

	public static void quad( int data ) {
		instrn( "quad", "0x" + Integer.toHexString( data ) );
		}

	public static void asciiz( String data ) {
		instrn( "asciiz", "\"" + Convert.toString( data ) + "\"" );
		instrn( "align" );
		}

	public static void align() {
		instrn( "align" );
		}

	public static void importFile( String fileName ) {
		instrn( "import", "\"" + fileName + "\"" );
		}

	public static void labelDefn( String label ) {
		Print.compile().println( "%-" + label + ":%+" );
		}

	public static void publicLabelDefn( String label ) {
		Print.compile().println( "%-public " + label + ":%+" );
		}

	}
