package node;
import env.*;
import type.*;
import type.basicType.*;
import template.*;
import text.*;
import runEnv.*;

public class ProgramNode extends Node {

	private DeclStmtListNode declStmtList;
	private Env env;
	private int weight;
	RunEnv runEnv;
	
	public ProgramNode( DeclStmtListNode declStmtList ) {
		this.declStmtList = declStmtList;
		}

	public String toString() {
		return declStmtList + "%n";
		}
		
	public void genEnv() {
		env = new Env( Env.ENV_GLOBAL );
		env.setDecl( new Decl( "main", Decl.SECT_NONE, Decl.NOACCESS, null, null, null ) );
		declStmtList.genEnv( Decl.SECT_GLOBAL, env );
		}

	public void genDecl() {
		declStmtList.genDecl();
		}

	public void checkType() {
		try {
			declStmtList.checkType();
			}
		catch ( Error exception ) {
			Print.error().println( exception.getMessage() );
			System.exit( -1 );
			}
		}

	public void eval() throws ReturnException {
		runEnv = new RunEnv( 
			new StaticValue( new Multiple( env.staticFieldOffset().value() ) ),
			null, null );
		declStmtList.eval( runEnv );
		}

	public void calcWeight() {
		weight = declStmtList.calcWeight();
		env.setMemberOffset();
		}

	public void genStaticCode() {
		int tempRegs = max( 0, weight - MAXREG );
		enterBlock( "static", null, null );
		enterBlock( "init", "proc.sav" + max( 0, weight - MAXREG ), "proc" );

		enter( "local" );
		publicLabelDefn( "max" );
		exit();

		enter( "code" );
		publicLabelDefn( "enter" );
		loadA( "$sp", displacement( "-max", "$sp" ) );
		store( "$ra", savRA( "$sp" ) );
		clear( "$ip" );
		clear( "$nip" );
		move( "$sp", "$fp" );
		declStmtList.genCode();
		load( "$ra", savRA( "$sp" ) );
		loadA( "$sp", displacement( "+max", "$sp" ) );
		ret();
		exit();

		exit();
		exit();
		}
	
	public void genCode() {
		instrn( "entry", "main.enter" );
		importFile( "../IMPORT/register.h" );
		importFile( "../IMPORT/callsys.h" );
		importFile( "../IMPORT/proc.h" );
		importFile( "../IMPORT/invoc.h" );
		importFile( "../IMPORT/callsys.lib.s" );
		importFile( "../IMPORT/memory.lib.s" ); // Keep outside lowData ... highData

		enterBlock( "lowData", null, null );
		enter( "data" );
		publicLabelDefn( "address" );
		exit();
		exit();
		
		importFile( "../IMPORT/string.lib.s" );
		importFile( "../IMPORT/number.lib.s" );
		importFile( "../IMPORT/bool.lib.s" );
		importFile( "../IMPORT/io.lib.s" );
		importFile( "../IMPORT/dString.lib.s" );
		importFile( "../IMPORT/array.lib.s" );
		importFile( "../IMPORT/INST.s" );

		enterBlock( "main", null, "register" );
		env.genClassDecls();
		env.genStaticDecls();
		genStaticCode();
		
		enter( "code" );
		publicLabelDefn( "enter" );
		clear( "$ip" );
		clear( "$nip" );

		loadIQ( "$a0", "lowData.address" );
		loadIQ( "$a1", "highData.address" );
		move( "$sp", "$a2" );
		bsr( "Memory.initialise.enter" );

		bsr( "static.init.enter" );

		move( "0", "$a0" );
		bsr( "Sys.exit.enter" );
		exit();

		exit();
		
		enterBlock( "highData", null, null );
		enter( "data" );
		publicLabelDefn( "address" );
		exit();
		exit();
		}
	}
