package node.declNode;
import node.*;
import node.declNode.extendsOptNode.*;
import node.declNode.sectionOptNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import runEnv.*;
import text.*;

public class ClassDeclNode extends DeclStmtNode implements DeclNode {

	private String ident;
	private ExtendsOptNode extendsOpt;
	private SectionOptNode staticBody;
	private SectionOptNode instanceBody;

	private ClassType classType;
	private Decl decl;
	private Decl extendedDecl;
	private ClassDeclNode extendedDeclNode;
	private Env localEnv;
	private int staticWeight;
	private int instanceWeight;

	public String ident() { return ident; }
	public Decl decl() { return decl; }
	public int instanceWeight() { return instanceWeight; }

	public ClassDeclNode( 
		String ident,
		ExtendsOptNode extendsOpt,
		SectionOptNode staticBody,
		SectionOptNode instanceBody ) {
		this.ident = ident;
		this.extendsOpt = extendsOpt;
		this.staticBody = staticBody;
		this.instanceBody = instanceBody;
		}
	
	public String toString() {
		return "class " + ident + " " + extendsOpt
			+ "%nbegin%+%n" 
			+ staticBody 
			+ instanceBody 
			+ "end%-";
		}
		
	public void genEnv( int section, Env env ) {
		Env extendedEnv;
		extendsOpt.genEnv( env );
		extendedDecl = extendsOpt.decl();
		if ( extendedDecl == null ) {
			extendedDeclNode = null;
			extendedEnv = null;
			}
		else {
			ClassType extendedClass = ( ClassType ) extendedDecl.type();
			extendedDeclNode = ( ClassDeclNode ) extendedDecl.declNode();
			extendedEnv = extendedClass.env();
			}
		localEnv = new Env( Env.ENV_CLASS, env, extendedEnv );
		classType = new ClassType( ident, localEnv );
		decl = env.declare( ident, Decl.SECT_CLASS, Decl.PUBLIC, classType, this );
		localEnv.setDecl( decl );
		staticBody.genEnv( localEnv );
		instanceBody.genEnv( localEnv );
		}

	public void genDecl() {
		staticBody.genDecl();
		instanceBody.genDecl();
		}

	public void checkType() {
		staticBody.checkType();
		instanceBody.checkType();
		}

	public void eval( RunEnv runEnv ) throws ReturnException {
		staticBody.eval( runEnv );
		}

	public void initInstance( RunEnv instanceRunEnv ) {
		if ( extendedDecl != null )
			extendedDeclNode.initInstance( instanceRunEnv );
		try {
			instanceBody.eval( instanceRunEnv );
			}
		catch ( ReturnException exception ) {
			error( "Illegal return within instance initialisation code" );
			}
		}

	public int calcWeight() {
		staticWeight = staticBody.calcWeight();
		instanceWeight = instanceBody.calcWeight();
		localEnv.setMemberOffset();
		localEnv.createMethodTable();
		return 0;
		}

	public void genStaticCode() {
		enterBlock( "static", null, null );
		enterBlock( "init", "proc.sav" + max( 0, staticWeight - MAXREG ), "proc" );

		enter( "local" );
		publicLabelDefn( "max" );
		exit();

		enter( "code" );
		align();
		publicLabelDefn( "enter" );
		loadA( "$sp", displacement( "-max", "$sp" ) );
		store( "$ra", displacement( "savRA", "$sp" ) );
		store( "$fp", displacement( "savFP", "$sp" ) );
		store( "$ip", displacement( "savIP", "$sp" ) );
		move( "$nip", "$ip" );
		move( "$sp", "$fp" );
		labelDefn( "body" );
		staticBody.genCode();
		labelDefn( "return" );
		move( "$ip", "$nip" );
		load( "$ip", displacement( "savIP", "$sp" ) );
		load( "$fp", displacement( "savFP", "$sp" ) );
		load( "$ra", displacement( "savRA", "$sp" ) );
		loadA( "$sp", displacement( "+max", "$sp" ) );
		ret();
		exit();
		exit();
		exit();
		}

	public void genInstanceCode() {
		enterBlock( "instance", null, null );
		enterBlock( "init", "proc.sav" + max( 0, instanceWeight - MAXREG ), "proc" );

		enter( "local" );
		publicLabelDefn( "max" );
		exit();

		enter( "code" );
		align();
		publicLabelDefn( "enter" );
		loadA( "$sp", displacement( "-max", "$sp" ) );
		store( "$ra", displacement( "savRA", "$sp" ) );
		store( "$fp", displacement( "savFP", "$sp" ) );
		store( "$ip", displacement( "savIP", "$sp" ) );
		move( "$nip", "$ip" );
		move( "$sp", "$fp" );
		if ( extendedDecl != null )
			bsr( extendedDecl.declName() + ".instance.init.enter" );
		labelDefn( "body" );
		instanceBody.genCode();
		labelDefn( "return" );
		move( "$ip", "$nip" );
		load( "$ip", displacement( "savIP", "$sp" ) );
		load( "$fp", displacement( "savFP", "$sp" ) );
		load( "$ra", displacement( "savRA", "$sp" ) );
		loadA( "$sp", displacement( "+max", "$sp" ) );
		ret();
		exit();
		exit();
		exit();
		}

	public void genDeclCode() {
		if ( extendedDecl != null )
			enterBlock( decl.declName(), null, extendedDecl.declName() );
		else
			enterBlock( decl.declName(), null, null );

		localEnv.genObjectDecls();
		localEnv.genClassDecls();
		localEnv.genMethodTable();
		localEnv.genStaticDecls();
		genStaticCode();
		localEnv.genInstanceDecls();
		genInstanceCode();

		exit();
		}

	public void genCode() {
		enter();
		bsr( decl.absoluteName() + ".static.init.enter" );
		exit();
		}

	}
