package node.exprNode;
import node.*;
import node.exprNode.variableNode.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class AssignNode extends ExprNode {

	private VariableNode variable;
	private ExprNode expr;
	private Type varType;
	
	public AssignNode( VariableNode variable, ExprNode expr ) {
		this.variable = variable;
		this.expr = expr;
		precedence = PREC_ASSIGN;
		}
	
	public String toString() {
		return variable + " = " + expr;
		}

	public void genEnv( Env env ) {
		variable.genEnv( env );
		expr.genEnv( env );
		}

	public Type checkType() {
		varType = variable.checkType();
		Type exprType = expr.checkType();
		expr = expr.castTo( varType );
		type = VoidType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunData address = variable.eval( runEnv );
		RunValue value = expr.eval( runEnv );
		address.setValue( value );
		return new NullValue();
		}
		
	public int calcWeight() {
		int variableWeight = variable.calcWeight();
		int exprWeight = expr.calcWeight();
		if ( variableWeight < MAXREG || exprWeight < MAXREG )
			weight = min( 
				max( variableWeight + 1, exprWeight ),
				max( exprWeight + 1, variableWeight ) );
		else
			weight = max( variableWeight + 1, exprWeight );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		int variableWeight = variable.weight();
		int exprWeight = expr.weight();
		if ( variableWeight < MAXREG - freeReg ) {
			expr.regCode( freeReg, freeTemp );
			String address = variable.addrCode( freeReg + 1, freeTemp );
			if ( varType instanceof RealType )
				storeF( ftempReg( freeReg ), address );
			else
				store( tempReg( freeReg ), address );
			}
		else if ( exprWeight < MAXREG - freeReg ) {
			String address = variable.addrCode( freeReg, freeTemp );
			expr.regCode( freeReg + 1, freeTemp );
			if ( varType instanceof RealType ) {
				storeF( ftempReg( freeReg + 1 ), address );
				}
			else {
				store( tempReg( freeReg + 1 ), address );
				}
			}
		else {
			expr.memCode( freeTemp );
			String address = variable.addrCode( freeReg, freeTemp + 1 );
			if ( varType instanceof RealType ) {
				load( ftempReg( freeReg + 1 ), tempMem( freeTemp ) );
				storeF( ftempReg( freeReg + 1 ), address );
				}
			else {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				store( tempReg( freeReg + 1 ), address );
				}
			}
		}
	}
