package node.exprNode;
import node.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public abstract class ExprNode extends Node {

	public final static int PREC_ASSIGN		=	2;
	public final static int PREC_OR			=	4;
	public final static int PREC_AND		=	5;
	public final static int PREC_EQ			=	9;
	public final static int PREC_REL		=	9;
	public final static int PREC_ADD		=	12;
	public final static int PREC_MUL		=	13;
	public final static int PREC_CAST		=	15;
	public final static int PREC_PREFIX		=	15;
	public final static int PREC_POSTFIX	=	16;
	public final static int PREC_PRIMARY	=	17;

	protected int precedence;
	protected Type type;
	public Type type() { return type; }
	protected int weight;
	public int weight() { return weight; }

	public String toString( int parentPrec ) {
		if ( precedence < parentPrec )
			return "( " + toString() + " )";
		else
			return toString();
		}

	public abstract void genEnv( Env env );
	public abstract Type checkType();
	public abstract RunValue eval( RunEnv runEnv );
	public abstract int calcWeight();

	public ExprNode castTo( Type postType ) {
		if ( postType == null )
			error( "Compiler Error - null type in castTo" );
		if ( type.equals( postType ) )
			return this;
		else if ( postType.isCastable( type ) )
			return new CastNode( postType, this );
		error( "Can't cast " + type + " to " + postType );
		return null;
		}
		
	public void memCode( int freeTemp ) {
		regCode( 0, freeTemp );
		if ( type instanceof RealType )
			storeF( ftempReg( 0 ), tempMem( freeTemp ) );
		else
			store( tempReg( 0 ), tempMem( freeTemp ) );
		}

	public void boolCode( 
		int freeReg, 
		int freeTemp, 
		String trueLabel, 
		String falseLabel, 
		boolean flowIntoTrue ) {
		if ( ! ( type instanceof BoolType ) )
			error( "Invoking boolCode for non boolean expression" );
		regCode( freeReg, freeTemp );
		if ( flowIntoTrue )
			instrn( "beq", tempReg( freeReg ), falseLabel );
		else
			instrn( "bne", tempReg( freeReg ), trueLabel );
		}

	public void regCode( int freeReg, int freeTemp ) {
		enter();
		labelDefn( "if" );
		boolCode( freeReg, freeTemp, "then", "else", true );
		labelDefn( "then" );
		move( "1", tempReg( freeReg ) );
		br( "end" );
		labelDefn( "else" );
		move( "0", tempReg( freeReg ) );
		labelDefn( "end" );
		exit();
		}
	}
