package node.exprNode.binaryNode.arithNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class PlusNode extends ArithNode {

	public PlusNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_ADD;
		operator = "+";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		if ( leftType instanceof StringType
			|| rightType instanceof StringType )
			type = StringType.type;
		else
			type = Type.balanceType( leftType, rightType );
		if ( type == null )
			error( "Incompatible operand types of " + leftType + 
				" and " + rightType + " for +" );
		left = left.castTo( type );
		right = right.castTo( type );
		if ( type instanceof StringType )
			opCode = "Array.concat.enter";
		else if ( type instanceof IntType )
			opCode = "addq";
		else if ( type instanceof RealType )
			opCode = "addt";
		operandType = type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( type instanceof IntType )
			return new IntValue( leftValue.intValue() + rightValue.intValue() );
		else if ( type instanceof RealType )
			return new RealValue( leftValue.realValue() + rightValue.realValue() );
		else if ( type instanceof StringType )
			return new StringValue( "" + leftValue + rightValue );
		error( "Invalid Type for +" );
		return null;
		}

	public int calcWeight() {
		int leftWeight = left.calcWeight();
		int rightWeight = right.calcWeight();
		if ( type instanceof StringType )
			weight = min( 
				max( leftWeight + 1, rightWeight, MAXREG ),
				max( rightWeight + 1, leftWeight, MAXREG ) );
		else
			weight = min( 
				max( leftWeight + 1, rightWeight ),
				max( rightWeight + 1, leftWeight ) );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		int leftWeight = left.weight();
		int rightWeight = right.weight();
		if ( rightWeight < MAXREG - freeReg ) {
			left.regCode( freeReg, freeTemp );
			right.regCode( freeReg + 1, freeTemp );
			if ( type instanceof StringType ) {
				move( tempReg( freeReg ), "$a0" );
				move( tempReg( freeReg + 1 ), "$a1" );
				bsr( "Array.concat.enter" );
				move( "$v0", tempReg( freeReg ) );
				}
			else if ( type instanceof RealType ) {
				instrn( opCode, ftempReg( freeReg ), 
					ftempReg( freeReg + 1 ), ftempReg( freeReg ) );
				}
			else {
				instrn( opCode, tempReg( freeReg ), 
					tempReg( freeReg + 1 ), tempReg( freeReg ) );
				}
			}
		else if ( leftWeight < MAXREG - freeReg ) {
			right.regCode( freeReg, freeTemp );
			left.regCode( freeReg + 1, freeTemp );
			if ( type instanceof StringType ) {
				move( tempReg( freeReg + 1 ), "$a0" );
				move( tempReg( freeReg ), "$a1" );
				bsr( "Array.concat.enter" );
				move( "$v0", tempReg( freeReg ) );
				}
			else if ( type instanceof RealType ) {
				instrn( opCode, ftempReg( freeReg + 1 ), 
					ftempReg( freeReg ), ftempReg( freeReg ) );
				}
			else {
				instrn( opCode, tempReg( freeReg + 1 ), 
					tempReg( freeReg ), tempReg( freeReg ) );
				}
			}
		else if ( rightWeight <= leftWeight ) {
			left.memCode( freeTemp );
			right.regCode( freeReg, freeTemp + 1 );
			if ( type instanceof StringType ) {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				move( tempReg( freeReg + 1 ), "$a0" );
				move( tempReg( freeReg ), "$a1" );
				bsr( "Array.concat.enter" );
				move( "$v0", tempReg( freeReg ) );
				}
			else if ( type instanceof RealType ) {
				loadF( ftempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, ftempReg( freeReg + 1 ), 
					ftempReg( freeReg ), ftempReg( freeReg ) );
				}
			else {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, tempReg( freeReg + 1 ), 
					tempReg( freeReg ), tempReg( freeReg ) );
				}
			}
		else {
			right.memCode( freeTemp );
			left.regCode( freeReg, freeTemp + 1 );
			if ( type instanceof StringType ) {
				move( tempReg( freeReg ), "$a0" );
				load( "$a1", tempMem( freeTemp ) );
				bsr( "Array.concat.enter" );
				move( "$v0", tempReg( freeReg ) );
				}
			else if ( type instanceof RealType ) {
				loadF( ftempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, ftempReg( freeReg ), 
					ftempReg( freeReg + 1 ), ftempReg( freeReg ) );
				}
			else {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, tempReg( freeReg ), 
					tempReg( freeReg + 1 ), tempReg( freeReg ) );
				}
			}
		}
	}
