package node.exprNode.binaryNode.relationNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;
import text.*;

public class EqualNode extends RelationNode {

	public EqualNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_EQ;
		operator = "==";
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		if ( leftType.equals( rightType ) )
			operandType = leftType;
		else if ( ( leftType instanceof ArrayType || leftType instanceof InstanceType )
			&& rightType instanceof NullType )
			operandType = leftType;
		else if ( ( rightType instanceof ArrayType || rightType instanceof InstanceType )
			&& leftType instanceof NullType )
			operandType = rightType;
		else {
			operandType = Type.balanceType( leftType, rightType );
			if ( operandType == null )
				error( "Incompatible operand types of " + leftType + 
					" and " + rightType + " for ==" );
			left = left.castTo( operandType );
			right = right.castTo( operandType );
			}
		if ( operandType instanceof IntType )
			opCode = "cmpeq";
		else if ( operandType instanceof RealType )
			opCode = "cmpteq";
		else 
			opCode = "cmpeq";
		type = BoolType.type;
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue leftValue = left.eval( runEnv );
		RunValue rightValue = right.eval( runEnv );
		if ( operandType instanceof IntType )
			return new BoolValue( leftValue.intValue() == rightValue.intValue() );
		else if ( operandType instanceof CharType )
			return new BoolValue( leftValue.charValue() == rightValue.charValue() );
		else if ( operandType instanceof BoolType )
			return new BoolValue( leftValue.boolValue() == rightValue.boolValue() );
		else if ( operandType instanceof RealType )
			return new BoolValue( leftValue.realValue() == rightValue.realValue() );
		else if ( operandType instanceof ArrayType )
			return new BoolValue( leftValue.arrayValue() == rightValue.arrayValue() );
		else if ( operandType instanceof InstanceType )
			return new BoolValue( leftValue.instanceValue() == rightValue.instanceValue() );
		error( "Invalid Type for ==" );
		return null;
		}

	public void boolCode( 
		int freeReg, 
		int freeTemp, 
		String trueLabel, 
		String falseLabel, 
		boolean flowIntoTrue ) {
		condCode( freeReg, freeTemp );
		if ( flowIntoTrue ) {
			if ( operandType instanceof RealType )
				instrn( "fbeq", ftempReg( freeReg ), falseLabel );
			else
				instrn( "beq", tempReg( freeReg ), falseLabel );
			}
		else {
			if ( operandType instanceof RealType )
				instrn( "fbne", ftempReg( freeReg ), trueLabel );
			else
				instrn( "bne", tempReg( freeReg ), trueLabel );
			}
		}

	}
