package node.exprNode.binaryNode.relationNode;
import node.*;
import node.exprNode.*;
import node.exprNode.binaryNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public abstract class RelationNode extends BinaryNode {

	public RelationNode( ExprNode left, ExprNode right ) {
		super( left, right );
		precedence = PREC_REL;
		}

	public String toString() {
		return 
			left.toString( precedence + 1 ) 
			+ " " +  operator + " " 
			+ right.toString( precedence + 1 );
		}

	public Type checkType() {
		Type leftType = left.checkType();
		Type rightType = right.checkType();
		operandType = Type.balanceType( leftType, rightType );
		if ( operandType == null )
			error( "Incompatible operand types of " + leftType + 
				" and " + rightType + " for relation" );
		left = left.castTo( operandType );
		right = right.castTo( operandType );
		type = BoolType.type;
		return type;
		}

	public void condCode( int freeReg, int freeTemp ) {
		int leftWeight = left.weight();
		int rightWeight = right.weight();
		if ( rightWeight < MAXREG - freeReg ) {
			left.regCode( freeReg, freeTemp );
			right.regCode( freeReg + 1, freeTemp );
			if ( operandType instanceof RealType ) {
				instrn( opCode, ftempReg( freeReg ), 
					ftempReg( freeReg + 1 ), ftempReg( freeReg ) );
				}
			else {
				instrn( opCode, tempReg( freeReg ), 
					tempReg( freeReg + 1 ), tempReg( freeReg ) );
				}
			}
		else if ( leftWeight < MAXREG - freeReg ) {
			right.regCode( freeReg, freeTemp );
			left.regCode( freeReg + 1, freeTemp );
			if ( operandType instanceof RealType ) {
				instrn( opCode, ftempReg( freeReg + 1 ), 
					ftempReg( freeReg ), ftempReg( freeReg ) );
				}
			else {
				instrn( opCode, tempReg( freeReg + 1 ), 
					tempReg( freeReg ), tempReg( freeReg ) );
				}
			}
		else if ( rightWeight <= leftWeight ) {
			left.memCode( freeTemp );
			right.regCode( freeReg, freeTemp + 1 );
			if ( operandType instanceof RealType ) {
				loadF( ftempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, ftempReg( freeReg + 1 ), 
					ftempReg( freeReg ), ftempReg( freeReg ) );
				}
			else {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, tempReg( freeReg + 1 ), 
					tempReg( freeReg ), tempReg( freeReg ) );
				}
			}
		else {
			right.memCode( freeTemp );
			left.regCode( freeReg, freeTemp + 1 );
			if ( operandType instanceof RealType ) {
				loadF( ftempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, ftempReg( freeReg ), 
					ftempReg( freeReg + 1 ), ftempReg( freeReg ) );
				}
			else {
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				instrn( opCode, tempReg( freeReg ), 
					tempReg( freeReg + 1 ), tempReg( freeReg ) );
				}
			}
		}
	}
