package node.exprNode.postfixNode;
import node.*;
import node.exprNode.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class PostDecNode extends ExprNode {

	private String operator;
	private String opCode;
	private Type operandType;
	private VariableNode left;

	public PostDecNode( VariableNode left ) {
		this.left = left;
		operator = "--";
		precedence = PREC_POSTFIX;
		}

	public String toString() {
		return  left + " " + operator;
		}

	public void genEnv( Env env ) {
		left.genEnv( env );
		}

	public Type checkType() {
		Type leftType = left.checkType();
		if ( ! ( leftType instanceof IntType ) && ! ( leftType instanceof CharType ) )
			error( "Operand of -- operator must be an int or char" );
		opCode = "subq";
		type = leftType;
		return type;
		}
				
	public RunValue eval( RunEnv runEnv ) {
		RunData leftData = left.eval( runEnv );
		RunValue leftValue = leftData.getValue();
		if ( type instanceof IntType )
			leftData.setValue( new IntValue( leftValue.intValue() - 1 ) );
		else
			leftData.setValue( new CharValue( ( char ) ( leftValue.charValue() - 1 ) ) );
		return leftValue;
		}

	public int calcWeight() {
		int leftWeight = left.calcWeight();
		weight = max ( leftWeight, 3 );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		int leftWeight = left.weight();
		if ( leftWeight < MAXREG ) {
			String address = left.addrCode( freeReg + 1, freeTemp );
			load( tempReg( freeReg ), address );
			instrn( opCode, tempReg( freeReg ), "1", tempReg( freeReg + 2 ) );
			store( tempReg( freeReg + 2 ), address );
			}
		else {
			String address = left.addrCode( freeReg, freeTemp );
			load( tempReg( freeReg + 1 ), address );
			instrn( opCode, tempReg( freeReg + 1 ), "1", tempReg( freeReg + 2 ) );
			store( tempReg( freeReg + 2 ), address );
			move( tempReg( freeReg + 1 ), tempReg( freeReg ) );
			}
		}
	}
