package node.exprNode.prefixNode;
import node.*;
import node.exprNode.*;
import node.exprNode.variableNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class PreIncNode extends ExprNode {

	private String operator;
	private String opCode;
	private Type operandType;
	private VariableNode right;

	public PreIncNode( VariableNode right ) {
		this.right = right;
		operator = "++";
		precedence = PREC_PREFIX;
		}

	public String toString() {
		return operator + " " + right;
		}

	public void genEnv( Env env ) {
		right.genEnv( env );
		}

	public Type checkType() {
		Type rightType = right.checkType();
		if ( ! ( rightType instanceof IntType ) && ! ( rightType instanceof CharType ) )
			error( "Operand of ++ operator must be an int or char" );
		opCode = "addq";
		type = rightType;
		return type;
		}
		
	public RunValue eval( RunEnv runEnv ) {
		RunData rightData = right.eval( runEnv );
		RunValue rightValue = rightData.getValue();
		if ( type instanceof IntType )
			rightData.setValue( new IntValue( rightValue.intValue() + 1 ) );
		else
			rightData.setValue( new CharValue( ( char ) ( rightValue.charValue() + 1 ) ) );
		return rightData.getValue();
		}

	public int calcWeight() {
		int rightWeight = right.calcWeight();
		weight = max ( rightWeight, 2 );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		int rightWeight = right.weight();
		if ( rightWeight < MAXREG ) {
			String address = right.addrCode( freeReg + 1, freeTemp );
			load( tempReg( freeReg ), address );
			instrn( opCode, tempReg( freeReg ), "1" );
			store( tempReg( freeReg ), address );
			}
		else {
			String address = right.addrCode( freeReg, freeTemp );
			load( tempReg( freeReg + 1 ), address );
			instrn( opCode, tempReg( freeReg + 1 ), "1" );
			store( tempReg( freeReg + 1 ), address );
			move( tempReg( freeReg + 1 ), tempReg( freeReg ) );
			}
		}
	}
