package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class ArrayCreationNode extends PrimaryNode {

	private ExprNode sizeExpr;
	private TypeNode typeNode;

	public ArrayCreationNode( ExprNode sizeExpr, TypeNode typeNode ) {
		this.sizeExpr = sizeExpr;
		this.typeNode = typeNode;
		}

	public String toString() {
		return "new [ " + sizeExpr + " ]" + typeNode;
		}

	public void genEnv( Env env ) {
		sizeExpr.genEnv( env );
		typeNode.genEnv( env );
		}

	public Type checkType() {
		Type sizeExprType = sizeExpr.checkType();
		Type subType = typeNode.getType();
		sizeExpr = sizeExpr.castTo( IntType.type );
		if ( subType instanceof CharType )
			type = StringType.type;
		else
			type = new ArrayType( subType );
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		RunValue size = sizeExpr.eval( runEnv );
		if ( type instanceof StringType )
			return new StringValue( new Multiple( size.intValue() ) );
		else
			return new ArrayValue( new Multiple( size.intValue() ) );
		}

	public int calcWeight() {
		int sizeWeight = sizeExpr.calcWeight();
		weight = max( sizeWeight, MAXREG );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		sizeExpr.regCode( freeReg, freeTemp );
		move( tempReg( freeReg ), "$s0" );
		move( 16, "$a0" );
		bsr( "Memory.allocate.enter" );
		move( "$v0", "$s1" );
		instrn( "s8addq", "$s0", "0", "$a0" );
		bsr( "Memory.allocate.enter" );
		store( "$s0", displacement( 0, "$s1" ) );
		store( "$v0", displacement( 8, "$s1" ) );
		move( "$s1", tempReg( freeReg ) );
		}
	}
