package node.exprNode.primaryNode;
import node.*;
import node.exprNode.*;
import node.typeNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;
import runEnv.basicValue.*;

public class InitArrayCreationNode extends PrimaryNode {

	private TypeNode typeNode;
	private ExprListNode exprList;

	public InitArrayCreationNode( TypeNode typeNode, ExprListNode exprList ) {
		this.typeNode = typeNode;
		this.exprList = exprList;
		}

	public String toString() {
		if ( exprList.size() == 0 )
			return "new []" + typeNode + "{}";
		else
			return "new []" + typeNode + "{ " + exprList + " }";
		}

	public void genEnv( Env env ) {
		typeNode.genEnv( env );
		exprList.genEnv( env );
		}

	public Type checkType() {
		Type subType = typeNode.getType();
		exprList.checkType();
		exprList.castArray( subType );
		if ( subType instanceof CharType )
			type = StringType.type;
		else
			type = new ArrayType( subType );
		return type;
		}

	public RunValue eval( RunEnv runEnv ) {
		Multiple multiple = exprList.evalArray( runEnv );
		if ( type instanceof StringType )
			return new StringValue( multiple );
		else
			return new ArrayValue( multiple );
		}

	public int calcWeight() {
		int exprListWeight = exprList.calcWeight();
		weight = max( exprListWeight + 1, MAXREG );
		return weight;
		}

	public void regCode( int freeReg, int freeTemp ) {
		loadIQ( "$a0", literal( 8 * exprList.size() ) );
		bsr( "Memory.allocate.enter" );
		if ( exprList.weight() < MAXREG - freeReg ) {
			move( "$v0", tempReg( freeReg ) );
			for ( int i = 0; i < exprList.size(); i++ ) {
				ExprNode expr = exprList.elementAt( i );
				expr.regCode( freeReg + 1, freeTemp );
				if ( expr.type() instanceof RealType )
					storeF( ftempReg( freeReg + 1 ), 
						displacement( 8 * i, tempReg( freeReg ) ) );
				else
					store( tempReg( freeReg + 1 ), 
						displacement( 8 * i, tempReg( freeReg ) ) );
				}
			}
		else {
			store( "$v0", tempMem( freeTemp ) );
			for ( int i = 0; i < exprList.size(); i++ ) {
				ExprNode expr = exprList.elementAt( i );
				expr.regCode( freeReg, freeTemp + 1 );
				load( tempReg( freeReg + 1 ), tempMem( freeTemp ) );
				if ( expr.type() instanceof RealType )
					storeF( ftempReg( freeReg ), 
						displacement( 8 * i, tempReg( freeReg + 1 ) ) );
				else
					store( tempReg( freeReg ), 
						displacement( 8 * i, tempReg( freeReg + 1 ) ) );
				}
			load( tempReg( freeReg ), tempMem( freeTemp ) );
			}
		move( tempReg( freeReg ), "$s0" );
		move( 16, "$a0" );
		bsr( "Memory.allocate.enter" );
		loadIQ( "$s1", literal( exprList.size() ) );
		store( "$s1", displacement( 0, "$v0" ) );
		store( "$s0", displacement( 8, "$v0" ) );
		move( "$v0", tempReg( freeReg ) );
		}
	}
