package node.stmtNode;
import node.*;
import node.exprNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class ForStmtNode extends StmtNode {

	private ExprNode initial;
	private ExprNode cond;
	private ExprNode increment;
	private DeclStmtListNode stmtList;

	public ForStmtNode( 
		ExprNode initial, 
		ExprNode cond, 
		ExprNode increment, 
		DeclStmtListNode stmtList ) {
		this.initial = initial;
		this.cond = cond;
		this.increment = increment;
		this.stmtList = stmtList;
		}

	public String toString() {
		return "for " + initial + "; " + cond + "; " 
			+ increment + " do%+%n" + stmtList + "%-%nend";
		}

	public void genEnv( Env env ) {
		initial.genEnv( env );
		cond.genEnv( env );
		increment.genEnv( env );
		stmtList.genEnv( Decl.SECT_LOCAL, env );
		}

	public void checkType() {
		Type initialType = initial.checkType();
		initial = initial.castTo( VoidType.type );
		Type condType = cond.checkType();
		cond = cond.castTo( BoolType.type );
		Type incrementType = increment.checkType();
		increment = increment.castTo( VoidType.type );
		stmtList.checkType();
		}
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		for ( initial.eval( runEnv ); cond.eval( runEnv ).boolValue(); increment.eval( runEnv ) )
			stmtList.eval( runEnv );
		}

	public int calcWeight() {
		int initWeight = initial.calcWeight();
		int condWeight = cond.calcWeight();
		int incWeight = increment.calcWeight();
		int stmtListWeight = stmtList.calcWeight();
		weight = max( initWeight, condWeight, incWeight, stmtListWeight );
		return weight;
		}

	public void genCode() {
		enter();
		labelDefn( "for" );
		initial.regCode( 0, 0 );
		labelDefn( "while" );
		cond.boolCode( 0, 0, "do", "end", true );
		labelDefn( "do" );
		stmtList.genCode();
		labelDefn( "continue" );
		increment.regCode( 0, 0 );
		br( "while" );
		labelDefn( "break" );
		labelDefn( "end" );
		exit();
		}
	}
