package node.stmtNode;
import node.*;
import node.exprNode.*;
import node.declNode.*;
import env.*;
import type.*;
import type.basicType.*;
import runEnv.*;

public class ReturnExprStmtNode extends StmtNode {

	private ExprNode expr;
	private Env env;
	private Decl methodDecl;
	private MethodDeclNode methodDeclNode;
	private MethodType methodType;
	private Type returnType;

	public ReturnExprStmtNode( ExprNode expr ) {
		this.expr = expr;
		}

	public String toString() {
		return "return " + expr +";";
		}

	public void genEnv( Env env ) {
		if ( ! ( env instanceof Env ) )
			error( "Attempting to return when not inside method" );
		this.env = ( Env ) env;
		expr.genEnv( env );
		}

	public void checkType() {
		Type exprType = expr.checkType();
		methodDecl = env.methodDecl();
		if ( methodDecl == null )
			error( "Return not inside method" );
		methodDeclNode = ( MethodDeclNode ) methodDecl.declNode();
		methodType = ( MethodType ) methodDecl.type();
		returnType = methodType.returnType();
		expr = expr.castTo( returnType );
		}
		
		
	public void eval( RunEnv runEnv ) throws ReturnException {
		RunValue result = expr.eval( runEnv );
		throw new ReturnException( result );
		}

	public int calcWeight() {
		int exprWeight = expr.calcWeight();
		weight = exprWeight;
		return weight;
		}

	public void genCode() {
		enter();
		expr.regCode( 0, 0 );
		br( "return" );
		exit();
		}
	}
