package grammar;

import java.io.*;
import java_cup.runtime.*;

%%

%public
%char
%type		Symbol

%{
	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType ) {
		System.err.println( "Obtain token " 
			+ sym.terminal_name( tokenType ) + " \"" + yytext() + "\"" );
		return new Symbol( tokenType, yychar, 
			yychar + yytext().length(), yytext() );
		}

%}

%init{
	yybegin( NORMAL );
%init}

number		=	[0-9]+
ident		=	[A-Za-z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n

%state NORMAL LEXERROR

%%

<NORMAL> {
	"="			{ return token( sym.ASSIGN ); }
	"+"			{ return token( sym.PLUS ); }
	"-"			{ return token( sym.MINUS ); }
	"*"			{ return token( sym.STAR ); }
	"/"			{ return token( sym.DIVIDE ); }
	"++"		{ return token( sym.INCR ); }
	"--"		{ return token( sym.DECR ); }
	"&"			{ return token( sym.AMPERSAND ); }
	","			{ return token( sym.COMMA ); }
	"("			{ return token( sym.LEFT ); }
	")"			{ return token( sym.RIGHT ); }
	{newline}	{ lineNumber++; return token( sym.NEWLINE ); }
	{space}		{ }

	{number}	{ return token( sym.NUMBER ); }
	{ident}		{ return token( sym.IDENT ); }

	.			{
				yybegin( LEXERROR ); 
				return token( sym.error ); 
				}
	}
	
<LEXERROR> {
	{newline}	{ 
				lineNumber++;
				yybegin( NORMAL ); 
				return token( sym.NEWLINE ); 
				}
	.			{ }
	}

<<EOF>>			{ return token( sym.EOF ); }

